@@ -5,8 +5,11 @@ import shlex
 import shutil
 from logger_tt import logger
 from multiprocessing import current_process
-import json
+
+# import json
+from collections import Counter
 from metaDMG.loggers.loggers import setup_logger
+from metaDMG.errors import metadamageError, Error
 from metaDMG.fit import mismatches, fits, results
 
 #%%
@@ -43,12 +46,11 @@ def data_dir(config, name, suffix="parquet"):
 
 def get_LCA_command(config):
     sample = config["sample"]
-    bam = config["samples"][sample]
     lca_rank = f"-lca_rank {config['lca_rank']}" if config["lca_rank"] != "" else ""
 
     command = (
         f"{config['metaDMG-lca']} lca "
-        f"-bam {bam} "
+        f"-bam {config['bam']} "
         f"-outnames {sample} "
         f"-names {config['names']} "
         f"-nodes {config['nodes']} "
@@ -81,6 +83,64 @@ def get_LCA_mismatches_command(config):
     return command[:-1]
 
 
+#%%
+
+
+def move_files(config):
+    sample = config["sample"]
+
+    d_move_source_target = {
+        f"{sample}.bdamage.gz.uglyprint.mismatch.txt": config["path_mismatches_txt"],
+        f"{sample}.bdamage.gz.uglyprint.stat.txt": config["path_mismatches_stat"],
+        f"{sample}.lca": config["path_lca"],
+        f"{sample}.log": config["path_lca_log"],
+    }
+    for source_path, target_path in d_move_source_target.items():
+        logger.debug(f"Moving {source_path} to {target_path}.")
+        if not Path(source_path).is_file():
+            raise metadamageError(f"{source_path} does not exist.")
+        Path(target_path).parent.mkdir(parents=True, exist_ok=True)
+        shutil.move(source_path, target_path)
+
+
+def delete_files(config):
+    sample = config["sample"]
+
+    bam = Path(config["bam"]).stem  # .name
+
+    paths_to_remove = [
+        f"{sample}.lca.stat",
+        f"{sample}.bdamage.gz",
+        *list(Path(".").glob(f"*{bam}*.bin")),
+    ]
+    for path in paths_to_remove:
+        logger.debug(f"Removing {path}.")
+        if not Path(path).is_file():
+            raise metadamageError(f"{path} does not exist.")
+        Path(path).unlink()
+
+
+def delete_all_sample_files(config):
+    sample = config["sample"]
+
+    bam = Path(config["bam"]).stem  # .name
+
+    paths_to_remove = [
+        f"{sample}.lca",
+        f"{sample}.lca.stat",
+        f"{sample}.log",
+        f"{sample}.bdamage.gz",
+        *list(Path(".").glob(f"*{bam}*.bin")),
+    ]
+    for path in paths_to_remove:
+        if Path(path).is_file():
+            logger.debug(f"Removing {path}.")
+            Path(path).unlink()
+
+
+#%%
+
+
 def run_command(command):
 
     p = subprocess.Popen(
@@ -95,35 +155,51 @@ def run_command(command):
                 line = line[:-1]
             yield line
 
-
-def move_files(config):
-    sample = config["sample"]
-
-    d_move_source_target = {
-        f"{sample}.bdamage.gz.uglyprint.mismatch.txt": config["path_mismatches_txt"],
-        f"{sample}.bdamage.gz.uglyprint.stat.txt": config["path_mismatches_stat"],
-        f"{sample}.lca": config["path_lca"],
-    }
-    for source_path, target_path in d_move_source_target.items():
-        logger.debug(f"Moving {source_path} to {target_path}.")
-        Path(target_path).parent.mkdir(parents=True, exist_ok=True)
-        shutil.move(source_path, target_path)
+    # waits for the process to finish and returns the returncode
+    yield p.wait()
 
 
-def delete_files(config):
-    sample = config["sample"]
+def handle_returncode(command, line, counter):
 
-    bam = Path(config["samples"][sample]).stem  # .name
+    command_string = " ".join(command.split()[:2])
 
-    paths_to_remove = [
-        f"{sample}.lca.stat",
-        f"{sample}.bdamage.gz",
-        f"{sample}.log",
-        *list(Path(".").glob(f"*{bam}*.bin")),
-    ]
-    for path in paths_to_remove:
-        logger.debug(f"Removing {path}.")
-        Path(path).unlink()
+    returncode = line
+    if returncode != 0:
+        s = f"{command_string} did not terminate properly."
+        raise metadamageError(s)
+
+    lines_hidden = {key: val for key, val in counter.items() if val >= 3}
+    if len(lines_hidden) > 0:
+        logger.debug(f"Hid the following lines: {lines_hidden}.")
+    logger.debug(f"Got return code {returncode} from {command_string}.")
+    return None
+
+
+def run_command_helper(config, command):
+
+    # add a counter to avoid too many similar lines
+    counter = Counter()
+    for line in run_command(command):
+
+        # if finished, check returncode
+        if isinstance(line, int):
+            return handle_returncode(command, line, counter)
+
+        # continue running and logging
+        if counter[line] < 3:
+            logger.debug(line)
+
+        # do not print the same line more than 3 times
+        elif counter[line] == 3:
+            # -> Problem finding level for rank: serotype
+            logger.debug("	...")
+            logger.debug("	...")
+            logger.debug("	...")
+
+        counter[line] += 1
+
+
+#%%
 
 
 def run_LCA(config, forced=False):
@@ -143,11 +219,10 @@ def run_LCA(config, forced=False):
         command_LCA_mismatches = get_LCA_mismatches_command(config)
 
         logger.debug(command_LCA)
-        for line in run_command(command_LCA):
-            logger.debug(line)
+        run_command_helper(config, command_LCA)
+
         logger.debug(command_LCA_mismatches)
-        for line in run_command(command_LCA_mismatches):
-            logger.debug(line)
+        run_command_helper(config, command_LCA_mismatches)
 
         move_files(config)
         delete_files(config)
@@ -252,44 +327,6 @@ def get_df_results(config, df_mismatches, df_fit_results, forced=False):
 #%%
 
 
-# class DB:
-#     def __init__(self, path):
-#         self.path = path
-
-#     def save(self, d):
-#         Path(self.path).parent.mkdir(parents=True, exist_ok=True)
-#         with open(self.path, "w") as write_file:
-#             json.dump(d, write_file, indent=4)
-
-#     def load(self):
-#         with open(self.path, "r") as read_file:
-#             d = json.load(read_file)
-#         return {int(k): v for k, v in d.items()}
-
-
-# def get_database_read_ids(config, forced=False):
-
-#     logger.info(f"Getting read_ids_mapping.")
-
-#     target = data_dir(config, name="database", suffix="json")
-
-#     database = DB(target)
-
-#     if do_run(target, forced=forced):
-#         logger.info(f"Computing read_ids_mapping.")
-#         read_ids_mapping = results.get_database_read_ids(config)
-#         database.save(read_ids_mapping)
-
-#     else:
-#         logger.info(f"Loading read_ids_mapping.")
-#         read_ids_mapping = database.load()
-
-#     return read_ids_mapping
-
-
-#%%
-
-
 def run_single_config(config):
 
     # if not main process (and haven't been initialized before)
@@ -302,12 +339,27 @@ def run_single_config(config):
 
     current_process().name = config["sample"]
 
-    run_LCA(config)
+    if not Path(config["bam"]).is_file():
+        logger.error(f"The sample bam file does not exist: {config['bam']}.")
+        return None
+
+    try:
+        run_LCA(config)
+    except metadamageError:
+        logger.exception(
+            f"{config['sample']} | metadamageError with run_LCA. "
+            f"See log-file for more information"
+        )
+        return None
+    except KeyboardInterrupt:
+        logger.info("Got KeyboardInterrupt. Cleaning up.")
+        delete_all_sample_files(config)
+        return None
+
     df_mismatches = get_df_mismatches(config)
     df_fit_results = get_df_fit_results(config, df_mismatches)
     df_results = get_df_results(config, df_mismatches, df_fit_results)
     # read_ids_mapping = get_database_read_ids(config)
 
     logger.info("Finished.")
-
     return df_mismatches, df_fit_results, df_results
